# -*- coding: utf-8 -*-
"""It is a technical analysis library to financial time series datasets.
You can use it to do feature engineering from financial datasets. It is builded
on pandas python library.

.. moduleauthor:: Dario Lopez Padial (Bukosabino)

"""
from .wrapper import *
from .utils import *
